/*
 * Copyright 2000, International Business Machines Corporation and others.
 * All Rights Reserved.
 *
 * This software has been released under the terms of the IBM Public
 * License.  For details, see the LICENSE file in the top-level source
 * directory or online at http://www.openafs.org/dl/license10.html
 */

#include <afs/afs_lock.h>
#include <ubik.h>
#include "ptint.h"
#include "ptserver.h"

#define pr_ErrorMsg afs_error_message
